
////////////////////////////////////////////////////////////////
//          SmartTemp             v6.004                      //
//       TFG Alejandro Escobar Fuertes                        //
//   SMARTTEMP (ESP32 + DS18B20 + Waveshare e-ink 1.54inches  //
//    + RTC + carga en EEPROM + NTP+ Relé Estado Solido)  +   //
//     Conexión a CloudMQTT                                   //
////////////////////////////////////////////////////////////////

#include <GxEPD.h>
#include <GxGDEP015OC1/GxGDEP015OC1.cpp>
#include <GxIO/GxIO_SPI/GxIO_SPI.cpp>
#include <GxIO/GxIO.cpp>//
#include <OneWire.h>
#include <DallasTemperature.h>
#include <WiFi.h>
#include "BitmapGraphics.h"
#include <NTPtimeESP.h>
#include <Wire.h>
#include <RTClib.h>
#include <EEPROM.h>
#include <PubSubClient.h>
#include <esp_task_wdt.h>
#include <esp_int_wdt.h>


#include <Fonts/FreeSansBold9pt7b.h>
#include <Fonts/FreeSansBold12pt7b.h>
#include <Fonts/FreeSansBold18pt7b.h>
#include <Fonts/FreeSansBold24pt7b.h>

#define ONE_WIRE_BUS 15

/*Definición del tamaño de la memoria EEPROM*/
#define EEPROM_SIZE 1024

/*Definición  tipo de RTC instalado y matriz de variable con los dias de la semana en texto */
RTC_DS3231 rtc;
char daysOfTheWeek[7][12] = {"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"};

/*Configuración entrada digital y salida digital*/
const int digitalIn0 = 4; //Numero de pin donde se ha cableado el pulsador Menu
const int digitalIn1 = 13; //Numero de pin donde se ha cableado el pulsador + (incrementar)
const int digitalIn2 = 12; //Numero de pin donde se ha cableado el pulsador - (decrementar)
const int digitalIn3 = 14; //Numero de pin donde se ha cableado el pulsador Ok

/*Datos para conectarse a la red local*/
const char* ssid     = "WLAN_AEF";
const char* password = "Alex-6782";

/*Datos CloudMQTT*/
char mqttServer[50] ="18.130.85.93";
int mqttPort = 1833;
int mqttSSLPort = 8883;
String mqttUser = "smarttemp";
char mqttPassword[50] = "12345";
char Nodo[50];

/*Declaración estructura topicos*/
String pathTemperatura = "/"+ mqttUser + "/" + "temperatura";
String pathModo = "/"+ mqttUser + "/" + "modo";
String pathModoSetup = "/"+ mqttUser + "/"+ "setup/" + "modo";
String pathSetpoint = "/"+ mqttUser + "/" + "setpoint";
String pathSPSetup = "/"+ mqttUser + "/" + "setup/" +"setpoint";
String pathOffset = "/"+ mqttUser + "/" + "offset";
String pathOffsetSetup = "/"+ mqttUser + "/" + "setup/" +"offset";
String pathHyst = "/"+ mqttUser + "/" + "hysteresis";
String pathHystSetup = "/"+ mqttUser + "/" + "setup/" +"hysteresis";
String pathProgSetup = "/"+ mqttUser + "/" + "setup/" +"programacion";
String pathPlanifSetup = "/"+ mqttUser + "/" + "setup/" +"planificacion";
String pathLectura = "/"+ mqttUser+ "/" + "peticion";
String pathCaldera ="/"+ mqttUser+ "/" + "caldera";




WiFiClient espClient;
PubSubClient client(espClient);

/*Variables temperatura, offset del sensor*/
float tempC = 0;
//float tempF = 0;          //no se usa actualmente
float offset = 2.2;
float hysteresis = 0.3;
float tempMem = 0;
float tempSP = 22.0;
float tempDay = 21;
float tempNight = 16.0;

/*Valiables Generales*/
int numDisplay = 0; //0= pantalla general, 1=ajuste Temp, 2=ajustes1, 3=ajustes2, 4=ajustes3
int currentDisplay = 0;
int timeOut = 0;                    //Timeout para volver a pantalla inicial
bool buttonState = false;           //variable para leer el estado del pulsador menu
bool button1State = false;          //variable para leer el estado del pulsador +
bool button2State = false;          //variable para leer el estado del pulsador -
bool button3State = false;          //variable para leer el estado del pulsador OK
bool memPushBut = false;            //variable memoria estado anterior de la entrada digital
bool memButtonOk = false;           //variable memoria estado anterior pulsador OK
bool baseInc = false;               //Base incremento temp ajust 0=0.1 y 1=1.0
int var1 = 0;
float increment = 0.1;                    //incremento TempSp
int modo = 0;                       //0=Auto, 1=Day, 2=Night, 3=Off
String modoText = " ";
int periodo= 5000;                  //duración espera 5s para temp
bool temp=false;                    //estado temporizador 0 lectura de la  temperatura
unsigned long tiempoAnterior=0;     //guarda tiempo de referencia para comparar con temp
String releMem=" ";                  //memoria estado rele
bool firstScan=true;                //primer ciclo de scan para lectura de la temperatura
int periodo1= 15000;                  //duración espera 15seg para temp1
bool temp1=false;                    //estado temporizador
unsigned long tiempo1Anterior=0;     //guarda tiempo de referencia para comparar
bool firstScan1=true;                 // primer ciclo de scan para actualización tiempo
bool rtcUpdateRequired=true;         //variable para indicar que necesita actualizarse
bool eepromUpdateEnbled=true;        //Variable para indicar que la EEPROM debe ser actualizada
int horaComprobacion=2;               //Hora a la que se sincroniza el RTC con el NTP
bool RTCisValid=0;                    //Tag para indicar que el RTC es valido
bool wifiConnected=false;              //Tag indicar si está conectado a Wifi
int tiempoEsperaWifi=40;               //Tiempo de espera a que esté conectado el valor x 500ms
float tempMatrix[7][24]={             //Array bidimensional temperatura en funcion [7]dias de la semana  [24]horas
  {17.0, 17.0, 17.0, 17.0, 17.0, 17.0, 17.0, 17.0, 20.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 20.0, 20.0},
  {17.0, 17.0, 17.0, 17.0, 17.0, 17.0, 20.0, 20.0, 20.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 20.0, 20.0},
  {17.0, 17.0, 17.0, 17.0, 17.0, 17.0, 20.0, 20.0, 20.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 20.0, 20.0},
  {17.0, 17.0, 17.0, 17.0, 17.0, 17.0, 20.0, 20.0, 20.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 20.0, 20.0},
  {17.0, 17.0, 17.0, 17.0, 17.0, 17.0, 20.0, 20.0, 20.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 20.0, 20.0},
  {17.0, 17.0, 17.0, 17.0, 17.0, 17.0, 20.0, 20.0, 20.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 20.0, 20.0},
  {17.0, 17.0, 17.0, 17.0, 17.0, 17.0, 17.0, 17.0, 20.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 22.0, 20.0, 20.0}};

/*
 * NTP time
 */
NTPtime NTPch("ch.pool.ntp.org");   // Choose server pool as required
strDateTime dateTime;


    byte actualHour = 0;
    byte actualMinute =0;
    byte actualSecond = 0;
    int actualYear = 0;
    byte actualMonth = 0;
    byte actualDay = 0;
    byte actualDayOfWeek = 0;

/*Sensor de temperatura DS18B20*/
OneWire oneWire(ONE_WIRE_BUS);
DallasTemperature sensors(&oneWire);
// ESP32 PINOUT (E-ink display)
//CS -> 17
//MOSI (DIN) ->23
//SCK (CLK) ->18
//DC -> 16
//RST -> 5
//BUSY -> 19

/*Pantall WaveShare e-ink 1.54 inches*/
//GxIO_Class io(SPI, CS_PIN, DC_PIN,RST_PIN);
//GxEPD_Class display(io, RST_PIN, BUSY_PIN);

GxIO_Class io(SPI, 17, 16, 5);
GxEPD_Class display(io, 5, 19);


/*
 * Salidas digitales
 * pin GPO33 configurado como salida digital, uso para relé
 */
 int rele =33;             //se le asigna el Pin 33 a rele



 
/*
 * 
 * Código de configuración e inicialización diferentes componentes existentes
 * 
 */

void setup() {

  Serial.begin(115200);
  display.init();
  sensors.begin();          //Inicialización sensores

  display.drawExampleBitmap(gImage_LoadingBlack, 0, 0, 200, 200, GxEPD_BLACK);
  display.update();
  connectWifi();            //Configuración Wifi
  delay(3000);              //delay de 3sec para abrir la consola
 
   //Inicializacion del pulsador
  pinMode(digitalIn0, INPUT);

 //Inicializacion salida digital
  pinMode(rele, OUTPUT);
  digitalWrite(rele, HIGH);           //forzamos que no se active el rele durante el setup del sistema
  

/*
 * RTC
 */

if (! rtc.begin()) {
    Serial.println("Couldn't find RTC");
   hard_restart();                                          //reiniciar el sistema
  }
 
  if (rtc.lostPower()) {
   Serial.println("RTC lost power, lets set the time!");
    updateRTCfromNTP();                                 //Actualizar RTC con NTP al detectar el RTC una perdida de alimentación de backup
}

/*
 * EEPROM code
 */
  if (!EEPROM.begin(EEPROM_SIZE))
  {
    Serial.println("failed to initialise EEPROM"); delay(1000000);          //Cambiar delay por reboot 
  }
   
  Serial.print("EEPROM size = ");
  Serial.println(EEPROM_SIZE);
 
  /*
   * Inicializar Matriz de temperatura desde EEPROM
   */
   initMatrix();

   //Cargar pantalla principal
  if(wifiConnected==true){
  display.drawExampleBitmap(gImage_MainPageWifi, sizeof(gImage_MainPageWifi), GxEPD::bm_default | GxEPD::bm_partial_update);
  }else{
    display.drawExampleBitmap(gImage_MainPageNoWifi, sizeof(gImage_MainPageNoWifi), GxEPD::bm_default | GxEPD::bm_partial_update);
  }

  /*
   * Establecer conexión con MQTTserver
   */
   
   client.setServer(mqttServer, mqttSSLPort);
   client.setCallback(receivedCallback);

}

void loop() {

  if(!client.connected()){
    if(wifiConnected==true){
    reconnect();
  }
  }

client.loop();

  //comprobar estado pulsadores
  buttonState = digitalRead(digitalIn0);              //pulsador Menu
  button1State = digitalRead(digitalIn1);             //pulsador +
  button2State = digitalRead(digitalIn2);             //pulsador -
  button3State = digitalRead(digitalIn3);             //pulsador Ok

      
   if(temp1==false){
        tiempo1Anterior=millis();    //guardamos el tiempo actual como referencia
        temp1=true;
      }

    if((millis()>tiempo1Anterior+periodo1 && temp1==true)||firstScan1==true){
        printTime();
        checkTime();       //Comprobar tiempo actual para saber si se debe actualizar el RTC
        updateEEPROM();       //Actualizar EEPROM cada dia a las 04h
        firstScan1=false;    //indicar que no es el primerscan
        checkWifiStatus();      //Comprobar estado Wifi
 
        temp1=false;           //desactivo temporizador

        if (numDisplay == currentDisplay && numDisplay == 9 ) {
          showTime();                                                   //actualizar tiempo si estoy en la pantalla 9 
        }
      }


  if (buttonState == true && memPushBut == false && button3State==false) {
    memPushBut = true; //memorización flanco positivo pulsador
    timeOut = 0;     //reset Timeout
    Serial.println(" Flanco Positivo pulsador detectado...");
    if (numDisplay < 5) {
      numDisplay++;
      Serial.println("Num Pantalla a cargar:");
      Serial.println( numDisplay);
    } else {
      if (numDisplay == 5) {
        numDisplay = 0;
        Serial.println("Num Pantalla a cargar:");
        Serial.println(numDisplay);
      } else {
        numDisplay = numDisplay - 4;
        Serial.println("Num Pantalla a cargar:");
        Serial.println(numDisplay);
      }
    }
  }
  if (buttonState == false && memPushBut == true) {
    memPushBut = false; //memorización flanco negativo pulsador
    Serial.println(" Flanco negativo pulsador detectado...");
  }

  //Leer y actualizar temperatura en Main page
  if (numDisplay == currentDisplay && numDisplay == 0 && memPushBut == 0 ) {
    if(temp==false){
      tiempoAnterior=millis();    //guardamos el tiempo actual como referencia
      temp=true;
    }
  
    if((millis()>tiempoAnterior+periodo && temp==true)||firstScan==true){
      firstScan=false;    //indicar que no es el primer refresco de la temperatura
    readTemperature();
    updateTemperature();  //Llamar a rutina actualización temperatura
    temp=false;           //desactivo temporizador

    //publicarParamFloatValue(pathSetpoint, tempSP);           //publicar temperatura configurada como setpoint *****mover***
   
    }
    
  }

  //Configuración Temperatura SetPoint
  if (numDisplay == currentDisplay && numDisplay == 1 ) {
    Serial.print("tempSP = ");Serial.println(tempSP);
      tempSP = modifyValue(tempSP, 32.0, 14.0);
      updateChangeTemp();
    }

  //Acceso a pantalla Modo
  if (numDisplay == currentDisplay && numDisplay == 2 ) {
    gotoDisplayNum(6);
      }

  //Acceso a pantalla de Network (only display)
  if (numDisplay == currentDisplay && numDisplay == 3 ) {
     gotoDisplayNum(7);
     }

  //Acceso a pantalla de configuración Temperatura Offset
  if (numDisplay == currentDisplay && numDisplay == 4  ) {
    gotoDisplayNum(8);
     }

  //Acceso a pantalla Time (only display)
  if (numDisplay == currentDisplay && numDisplay == 5  ) {
    gotoDisplayNum(9);            
     }

  //Modificación modo en pantalla 6
  if (numDisplay == currentDisplay && numDisplay == 6  ) {
    bool modoUpdated = changeMode();
    if(modoUpdated==true){                          //si el modo ha sido variado entonces ejecutamos actualización de pantalla
          modoText = currentMode(modo);
          showText(modoText, 1, 36,16, 30, 65, 150, 90);
    }
  }

  //Pantalla 7 Network Settings
  if (numDisplay == currentDisplay && numDisplay == 7  ) {
      if(button3State==true && wifiConnected==false){
        timeOut=0;                                  //reseteamos el Timeout
         connectWifi();                             //reconectar an red Wifi
         Serial.print(" wifiConnected = "); Serial.println(wifiConnected);
         if(wifiConnected == true){
          Serial.println("refrescamos pantalla 7");
              changeDisplay(7);                    //refrescar pantalla
        }else{
          Serial.println("Volvemos a pantalla 4");
          changeDisplay(4);                          //retroceder a pantalla 4 si no se ha conectado a Wifi
        }
 
   }
  }
 

  //Modificación offset en pantalla 8
  //Configuración Temperatura SetPoint
  if (numDisplay == currentDisplay && numDisplay == 8 ) {
    offset = modifyValue(offset, 7, 0);
  }

 //Pantalla 9 Time Settings
    if (numDisplay == currentDisplay && numDisplay == 9  ) {
     if( button3State==true && wifiConnected==true){
      
         updateRTCfromNTP();             //Llamar a funcion actualizacion reloj
         if(RTCisValid == true){
              changeDisplay(9);         //actualizar pantalla 9 con la hora correcta
      }else{
        changeDisplay(5);               //retroceder a pantalla 5 si el RTC no es valido
   
      }   
   }
    }

  //Cargar nueva pantalla
  if (numDisplay != currentDisplay) {
    changeDisplay(numDisplay);         //subrutina change display
  }

  if (timeOut == 150) {
    numDisplay = 0;    //forzar al sistema a volver a la pantalla inicial
    timeOut = 0;       //reset Timeout
  }

  if (currentDisplay != 0) {
    temp=false;           //reseteamos el temporizador
    firstScan=true;      //Volvemos a poner a 1 el first scan para indicar que no estamos mostrandola temperatura en pantalla 1
    delay(100);           //este delay no se puede borrar
    timeOut++;            //incrementamos el contador de timeout cada 100milis
    if(buttonState==true && button3State==true){
     numDisplay = 0;    //forzar al sistema a volver a la pantalla inicial
      timeOut = 0;       //reset Timeout
    }
  }



}       //Bucle principal

/*
 * 
 * Función para la lectura de la temperatura y aplicación del offset 
 * 
 */
void readTemperature() {
  sensors.requestTemperatures();
  tempC = sensors.getTempCByIndex(0);
  tempC = tempC - offset;
 // tempF = sensors.toFahrenheit(tempC);     //No se usa actualmente lo dejo por si se quiere ampliar la posibilidad de mostrar en F 
 }

/*
 * Funcion restart ESP32
 */
void hard_restart() {
  esp_task_wdt_init(1,true);
  esp_task_wdt_add(NULL);
  while(true);
}


/*
 * 
 * Función que actualiza la temperatura si esta es diferente a la memorizada
 * Y si está dentro del rango de valores aceptados 0.1 y 49.9 grados
 * 
 */

void updateTemperature() {
  if (tempMem != tempC) {
    if(tempC>0 && tempC<50){          //limite de temperatura leida valida entre 0.1 y 49.9 grados
    tempMem = tempC;
    showTemp(tempC);                  //Llamada a mostrar temperatura por pantalla
    printTemperatureToSerial();
    }
  }
    
  relayControl();                     //Llamada a control del relé

}

/*
 * Función mostrar temp por pantalla
 * @temperature --- Valor de entrada con la temperatura adquirida
 */
void showTemp(float temperature)
{
  Serial.println("Llamada ShowTemp Routine...");
  String temperatureString;
  if (temperature > 100)
  {
    temperatureString = String(temperature, 0);
  } else
  {
    temperatureString = String(temperature, 1);
  }
  const char* name = "FreeSansBold24pt7b";
  const GFXfont* f = &FreeSansBold24pt7b;


  uint16_t box_x = 50;
  uint16_t box_y = 65;
  uint16_t box_w = 90;
  uint16_t box_h = 90;
  uint16_t cursor_y = box_y + 16;
  if (temperature > 10.0) {
    box_x = box_x + 0;
  } else {
    box_x = box_x + 10;
    box_w = box_w - 10;
  }
  Serial.println("Escribiendo en pantalla desde ShowTemp");
  display.setRotation(45);
  display.setFont(f);
  display.setTextColor(GxEPD_BLACK);

  display.fillRect(0, 0, 200, 200, GxEPD_WHITE);
  display.setCursor(box_x, cursor_y + 36);
  display.print(temperatureString);
  display.updateWindow(box_x, box_y, box_w, box_h, true);
}

/*
 * void showText(String text, int fontSize,uint16_t cursorX, uint16_t cursorY, uint16_t box_x, uint16_t box_y, uint16_t box_w, uint16_t box_h)
 * * @fontSize -- valor integer para indicar el tamaño del texto a mostrar
        0- FreeSansBold9pt7b
        1- FreeSansBold12pt7b
        2- FreeSansBold18pt7b
        3- FreeSansBold24pt7b
  * * @cursorX - valor integer para seleccionar la posición en X
  * * @cursorY - valor integer para seleccionar la posición en Y
  * * @box_X - valor integer para seleccionar la posición de la caja en X
  * * @box_Y - valor integer para seleccionar la posición de la caja en Y
  * * @box_w - valor integer para seleccionar el ancho de la caja 
  * * @box_h - valor integer para seleccionar el alto de la caja
  * 
  *  Funcion no retorna nigun valor
*/
void showText(String text, int fontSize,uint16_t cursorX, uint16_t cursorY, uint16_t box_x, uint16_t box_y, uint16_t box_w, uint16_t box_h)
{
  Serial.println("Llamada a showText Routine...");
  String dataString;
  const GFXfont* font = &FreeSansBold24pt7b;

  if (text == "Wifi") {
    dataString = WiFi.localIP().toString();
  } else {
    dataString = text;
  }

  switch (fontSize) {
    case 0:
      font = &FreeSansBold9pt7b;
      break;
    case 1:
      font = &FreeSansBold12pt7b;
      break;
    case 2:
      font = &FreeSansBold18pt7b;
      break;
    case 3:
      font = &FreeSansBold24pt7b;
      break;
  }

 uint16_t cursor_y = box_y + cursorY;
  Serial.println("Escribiendo en pantalla desde ShowText");
  display.setRotation(45);
  display.setFont(font);
  display.setTextColor(GxEPD_BLACK);

  display.fillRect(0, 0, 200, 200, GxEPD_WHITE);
  display.setCursor(box_x, cursor_y + cursorX);
  display.print(dataString);
  display.updateWindow(box_x, box_y, box_w, box_h, true);
}

/*
 * 
 * Función para cargar la pantalla gráfica en función al número de display actual
 * @displayNum ---- valor integer del display a cargar
 * 
 */

void changeDisplay(int displayNum) {
  currentDisplay = numDisplay;   //Cargar num de pantalla en memoria
  tempMem = 100;                   //reseteo temperatura memorizada

  Serial.println("Llamada change Display Routine...");
  Serial.print("NumDisplay = ");
  Serial.println(numDisplay);
  Serial.print("currentDisplay = ");
  Serial.println(currentDisplay);
  switch (numDisplay) {
    case 0:  //Main page
      Serial.println("Escribiendo en pantalla desde ChangeDisplay Case0");
      //Cargar pantalla principal
          checkWifiStatus();            //Comprobar el estado de la red
            if(wifiConnected==true){
                display.drawExampleBitmap(gImage_MainPageWifi, sizeof(gImage_MainPageWifi), GxEPD::bm_default | GxEPD::bm_partial_update);
            }else{
                display.drawExampleBitmap(gImage_MainPageNoWifi, sizeof(gImage_MainPageNoWifi), GxEPD::bm_default | GxEPD::bm_partial_update);
            }
      currentDisplay = numDisplay;
      break;
    case 1:  //Ajsute Temp setpoint
      Serial.println("Escribiendo en pantalla desde ChangeDisplay Case01");
      display.drawExampleBitmap(gImage_ajusteTemp, sizeof(gImage_ajusteTemp), GxEPD::bm_default | GxEPD::bm_partial_update);
      showTemp(tempSP);
      currentDisplay = numDisplay;
      break;
    case 2:  //Ajustes1
      Serial.println("Escribiendo en pantalla desde ChangeDisplay Case02");
      display.drawExampleBitmap(gImage_ajustes1, sizeof(gImage_ajustes1), GxEPD::bm_default | GxEPD::bm_partial_update);
      currentDisplay = numDisplay;
      break;
    case 3:  //Ajustes2
      Serial.println("Escribiendo en pantalla desde ChangeDisplay Case03");
      display.drawExampleBitmap(gImage_ajustes2, sizeof(gImage_ajustes2), GxEPD::bm_default | GxEPD::bm_partial_update);
      currentDisplay = numDisplay;
      break;
    case 4:  //Ajustes3
      Serial.println("Escribiendo en pantalla desde ChangeDisplay Case04");
      display.drawExampleBitmap(gImage_ajustes3, sizeof(gImage_ajustes3), GxEPD::bm_default | GxEPD::bm_partial_update);
      currentDisplay = numDisplay;
      break;
    case 5:  //Ajustes4
      Serial.println("Escribiendo en pantalla desde ChangeDisplay Case05");
      display.drawExampleBitmap(gImage_ajustes4, sizeof(gImage_ajustes4), GxEPD::bm_default | GxEPD::bm_partial_update);
      currentDisplay = numDisplay;
      break;
    case 6:  //Display mode
      Serial.println("Escribiendo en pantalla desde ChangeDisplay Case06");
      display.drawExampleBitmap(gImage_Mode, sizeof(gImage_Mode), GxEPD::bm_default | GxEPD::bm_partial_update);
      modoText = currentMode(modo);
      showText(modoText, 1, 36, 16, 30, 65, 150, 90);
      currentDisplay = numDisplay;
      break;
    case 7:  //Display networks settings
      Serial.println("Escribiendo en pantalla desde ChangeDisplay Case07");
      display.drawExampleBitmap(gImage_Network, sizeof(gImage_Network), GxEPD::bm_default | GxEPD::bm_partial_update);
      showText("Wifi", 1, 36, 16, 30, 65, 150, 90);
      currentDisplay = numDisplay;
      break;
    case 8:  //Ajuste Temp setpoint
      Serial.println("Escribiendo en pantalla desde ChangeDisplay Case08");
      display.drawExampleBitmap(gImage_ajusteTemp, sizeof(gImage_ajusteTemp), GxEPD::bm_default | GxEPD::bm_partial_update);
      showTemp(offset);
      currentDisplay = numDisplay;
      break;
    case 9:  //Display time
      Serial.println("Escribiendo en pantalla desde ChangeDisplay Case09");
      display.drawExampleBitmap(gImage_Time, sizeof(gImage_Time), GxEPD::bm_default | GxEPD::bm_partial_update);
      currentDisplay = numDisplay;
      showTime();
      break;
  }
}

/*
 * cargar modo de trabajo actual en modo texto
  * @ modeTemp; valor del modo actual
  * Valor de salida tipo String
*/

String currentMode(int modeTemp) {
  String modeLoad;
  Serial.println("Llamada currentMode Routine...");

  switch (modeTemp) {
    case 0:
      modeLoad = "Automatico";
      break;
    case 1:
      modeLoad = "Manual dia";
      break;
    case 2:
      modeLoad = "Manual noche";
      break;
    case 3:
      modeLoad ="Apagado";
     }
  loadTempMode(modo);               //Llamar rutina cargar temp en base al modo actual
  return modeLoad;
}

/*
* Funcion para sacar temperatura por puerto serie
*/

void printTemperatureToSerial()
{
  Serial.print(tempC);
  Serial.print(" C");
  Serial.print("\t");
 // Serial.print(tempF);      //no se usa actualmente Farenheit 
 // Serial.print(" F");       //no se usa actualmente
  Serial.print("\n");
  
   publicarParamFloatValue(pathTemperatura, tempC);        //publicar temperatura actual


}

/*
 * 
 * Función para sincronizar el ESP32 con el Router Wifi
 * 
 */

void connectWifi() {
  // We start by connecting to a WiFi network
  int numBucles=0;
  int numIntentos=0;

  Serial.println();
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);


  WiFi.begin(ssid, password);


  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");

    if(numBucles < tiempoEsperaWifi){ 
    numBucles++;                //incrementamos numero de bucles
    }else{
      numIntentos++;            //incrementamos intento fallido
      break;
    }
  checkWifiStatus();
}


}

/*
 * 
 * Funcion comprobar el estado actual de la Wifi
 * Si estamos en la pantalla 0 y el estado actual es diferente al anterior refrescamos pantalla 0
 * 
 */

void checkWifiStatus(){
  
if(WiFi.status() == WL_CONNECTED){
  Serial.println("");
  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
  if((wifiConnected==false) && (currentDisplay==0) ){
    wifiConnected=true;
    changeDisplay(0);
  }
  
}else{
  Serial.println("");
  Serial.println("WiFi not connected");
  if((wifiConnected==true) && (currentDisplay==0) ){
    wifiConnected=false;
    changeDisplay(0);
  }
}
}

/*
 * 
 * Función para cambiar la base de incremento
 * Por defecto está en 0.1 si pulsamos el botón de OK cambia a 1.0
 * Si volvemos a presionar vuelve a 0.1
 * 
 */

float modifyValue(float valueIn, float highLimit, float lowLimit) {
  Serial.println("Llamada modifyValue Routine...");
  if (button3State == true && baseInc == false) {
    baseInc = true;
    timeOut = 0;
    switch (var1) {
      case 0:
        increment = 1.0;
        var1 = 1;
        break;
      case 1:
        increment = 0.1;
        var1 = 0;
        break;
    }
  }

  if (button3State == false && baseInc == true) {
    baseInc = false;
  }
  if (button1State == true && button2State == false) {
    timeOut = 0;
    Serial.print("---Pulsador incrementar detectado...---"); Serial.print(" Temperatura SP = ");Serial.print(valueIn);Serial.print(" HighLimit = ");Serial.println(highLimit);
    if (valueIn <= highLimit ) {
      Serial.println("Dentro de bucle  if (valueIn <= highLimit )");
      valueIn = valueIn + increment;
      delay(10);
      showTemp(valueIn);
    }
  }
  if (button2State == true && button1State == false) {
    timeOut = 0;
     Serial.print("---Pulsador decrementar detectado...---"); Serial.print(" Temperatura SP = ");Serial.print(valueIn);Serial.print(" LowLimit = ");Serial.println(lowLimit);
    if (valueIn >= lowLimit) {
        Serial.println("Dentro de bucle  if (valueIn >= lowLimit)");
      valueIn = valueIn - increment;
      delay(10);
      showTemp(valueIn);
    }
  }
  return valueIn;
}


/*
 * 
 * Funcion para cambiar el modo de trabajo
 * 0 = Auto
 * 1 = Manual temperatura dia
 * 2 = Manual temperatura noche
 * 3 = OFF
 * 
 */

bool changeMode() {
  Serial.println("Llamada changeMode Routine...");
  int modoTemp= modo;               //guardamos estado modo inicial
  bool modoChanged=false;           //indicador modo trabajo ha cambiado
  if (button1State == true && button2State == false) {
    timeOut = 0;
    if (modo < 3) {
      modo++;
      delay(50);
    }else{
      modo=0;
      delay(50);
    }
  }
  if (button2State == true && button1State == false) {
    timeOut = 0;
    if (modo > 0) {
      modo--;
      delay(50);
    }else{
      modo=3;
      delay(50);
    }
  }

  if(modoTemp!=modo){               //comprobamos si el modo actual es diferente al inicial
    modoChanged=true;               //si son diferentes entonces ativamos modoChanged para indicar que el modo ha cambiado
  }
  return modoChanged;               //retornamos si el modo ha variado o no
}

/*
 * 
 * Función para el cambio de pantalla con el botón de cmabio de menú
 * 
 */

void gotoDisplayNum(int disPlayNum) {
  Serial.println("Llamada gotoDisplay Routine...");
  if (button3State == true && memButtonOk == false) {
    memButtonOk = true;
    timeOut = 0;
  }
  if (button3State == false && memButtonOk == true) {
    memButtonOk = false;
    numDisplay = disPlayNum;          //cargamos display de destino en cuanto detectamos el flanco negativo del pulsador
  }
  delay(10);
}


void loadTempMode(int modoTemp) {
  Serial.println("Llamada loadTempMode Routine...");
  int horaLoadTemp = getHora();
  int diaLoadTemp = getDia();
  switch (modoTemp){
    case 0:
       tempSP = getProgTemp(horaLoadTemp,diaLoadTemp);          //Aqui se cargaria la temperatura en base a la hora ya que el modo es automatico
      break;
    case 1:
       tempSP = tempDay;         //Cargar la temperatura Dia
      break;
    case 2:
       tempSP = tempNight;         //Cargar la temperatura Noche
      break;
    case 3:
      tempSP = 0.0;               //cargar 0.0 al setpoint para deshabilitar el funcionamiento
      break;
  }
  }

/*
 * 
 *  Función para mostrar por Monitor Serie los valores del RTC de forma concatenada
 *  Uso para debug
 * 
 */

void showTime(){

    DateTime now = rtc.now();
String actualTime = daysOfTheWeek[now.dayOfTheWeek()];
showText(actualTime,1, 20,20, 30, 65, 150, 50);
actualTime=String(now.hour());
actualTime=actualTime + ":";
actualTime= actualTime + String(now.minute());
Serial.println(actualTime);
showText(actualTime,1, 20,20, 45, 120, 150, 50);

}


/*
 * 
 *  Función para mostrar por Monitor Serie los valores del RTC 
 *  Uso para debug
 * 
 */
 
void printTime(){
  DateTime now = rtc.now();
  Serial.print(now.year());
    Serial.print('/');
    Serial.print(now.month());
    Serial.print('/');
    Serial.print(now.day());
    Serial.print(" (");
    Serial.print(now.dayOfTheWeek());
    Serial.print(" = ");
    Serial.print(daysOfTheWeek[now.dayOfTheWeek()]);
    Serial.print(") ");
    Serial.print(now.hour());
    Serial.print(':');
    Serial.print(now.minute());
    Serial.print(':');
    Serial.print(now.second());
    Serial.println();
}


/*
 * 
 * Función para comprobar el RTC
 * 
 */

void checkTime(){
   DateTime now = rtc.now();
/*
 * Comprobación del mes actual para configurar la hora de comprobación
 * Para el ajuste horario automatico en Primavera u Otoño
 */   
  if(now.month()==10 && horaComprobacion==2){
    horaComprobacion=3;
  }
    if(now.month()!=10 && horaComprobacion==3){
    horaComprobacion=2;
  }
   
   Serial.println("---checking time on CheckTime()---");
   if(now.dayOfTheWeek()==0 && rtcUpdateRequired==true){
     if(now.hour()==horaComprobacion){             //Si es la hora de comprobación 
       if(now.second()>1){                        //Si el segundo es superior a 1 para evitar actualizar en 00
        if(WiFi.status()== WL_CONNECTED){         //Si tiene conexión de red
       updateRTCfromNTP();
       rtcUpdateRequired=false;             //indicamos que el RTC ya ha sido actualizado
        }                        
      }
    }else{
      if( rtcUpdateRequired==false){
         rtcUpdateRequired=true;           //indicamos que el RTC ya puede ser actualizado de nuevo
      }
    }
   }
}

/*
 * Retorna el dia actual
 */
int getDia(){
   DateTime now = rtc.now();
   return now.dayOfTheWeek();
}

/*
 * Retorna la hora actual
 */
int getHora(){
   DateTime now = rtc.now();
   return now.hour();
}

/*
 * 
 * Función para actualizar el RTC con el NTP 
 * En el caso que WiFi.status() == WL_CONNECTED
 * De lo contrario no lo actualiza
 *
 */

void updateRTCfromNTP(){
     if(WiFi.status()== WL_CONNECTED){
   dateTime = NTPch.getNTPtime(1.0, 1);
   while(!dateTime.valid){
   delay(100);
   Serial.println("getting time from NTPserver for RTC adjust");
   dateTime = NTPch.getNTPtime(1.0, 1);
    }
    //dateTime.valid = true
    rtc.adjust(DateTime(dateTime.year,dateTime.month,dateTime.day,dateTime.hour,dateTime.minute,dateTime.second));
    Serial.println("****RTC adjusted*****");
    delay(50);
    printTime();      //sacar RTC time por consola
  }else{
   
   Serial.println("No Internet connection established No valid time available");
   RTCisValid=false;              //Indicamos que el RTC no es valido para trabajar sin RTC
  }
}


/*
 * subrutina para inicialización temperatura en función del dia y hora
 */
void initMatrix(){

int direccionInicio=0;

float varCheck=0.0;

//Comprobamos si la EEPROM está vacía o no

EEPROM.get(direccionInicio,varCheck);
Serial.print("EEPROM direccion ");
Serial.println(direccionInicio); 
Serial.print("valor que hemos obtenido de la EEPROM ");
Serial.println(varCheck);
  
if(varCheck>0.0){
      for ( int i = 0; i < 7; i++ ) {
      for ( int j = 0; j < 24; j++ ) {

        Serial.println("----------------leer---------");
        Serial.print("i= ");
        Serial.print(i);
        Serial.print(" j= ");
        Serial.print(j);
        Serial.print(" EEPROM Address ");
        
        Serial.print(direccionInicio);
        EEPROM.get(direccionInicio, tempMatrix[i][j]);  
        Serial.print(" i= ");
        Serial.print(i);
        Serial.print(" j= ");
        Serial.print(j);
        Serial.print(" , ");
        Serial.print(tempMatrix[i][j]);
        Serial.println(");");
        Serial.println("--------------------------------------------");
        
        
        direccionInicio = direccionInicio + sizeof(tempMatrix[i][j]);
        delay(5);
      }
   }

   //recuperamos valor offset
   
      EEPROM.get(direccionInicio, offset);
          Serial.println("-----------Lectura--------------");
          Serial.print("Valor de offset ");
          Serial.print(offset);
          Serial.print(" EEPROM Address ");
          Serial.print(direccionInicio);
      direccionInicio = direccionInicio + sizeof(offset);
      delay(5);

   //recuperamos valor tempDay
      EEPROM.get(direccionInicio, tempDay);
          Serial.println("-----------Lectura--------------");
          Serial.print("Valor de tempDay ");
          Serial.print(tempDay);
          Serial.print(" EEPROM Address ");
          Serial.print(direccionInicio);
      direccionInicio = direccionInicio + sizeof(tempDay);
      delay(5);

   //recuperamos valor tempNight
      EEPROM.get(direccionInicio, tempNight);
          Serial.println("-----------Lectura--------------");
          Serial.print("Valor de tempNight ");
          Serial.print(tempNight);
          Serial.print(" EEPROM Address ");
          Serial.print(direccionInicio);
      direccionInicio = direccionInicio + sizeof(tempNight);
      delay(5);
 
}else{
  Serial.println("EEPROM vacía, se continua con los valores por defecto en la Matriz");
}
}

/*
 * subrutina control del relé
 */
 void relayControl(){
 
  loadTempMode(modo);
  
   //Calcular limites
    float lowLimit = tempSP - hysteresis;
    float highLimit = tempSP + hysteresis;


    Serial.print("rele ==");
    Serial.println(digitalRead(rele));
      
      if(tempMem >= highLimit){
        if(digitalRead(rele==LOW)){
             digitalWrite(rele, HIGH);
             releMem="OFF";
             Serial.println("********Rele desactivado, temperatura deseada alcanzada *****");
         }else{
           Serial.println("---------------------------------------");
       Serial.println("tempMem >= highLimit pero Rele ya está desactivado");
      Serial.print("Temperatura actual: ");
      Serial.println(tempMem);
      Serial.print("Temperatura limite inferior: ");
      Serial.println(lowLimit);
      Serial.print("Temperatura limite superior: ");
      Serial.println(highLimit);
       Serial.println("--------------------------------------");
         }
        }       
       if(tempMem <= lowLimit){
          if(digitalRead(rele==HIGH)){
             digitalWrite(rele, LOW);
             releMem="ON";
             Serial.println("********Rele activado, temperatura por debajo a la deseada *****");
          }else{
                  Serial.println("---------------------------------------");
       Serial.println("tempMem <= highLimit pero Rele ya está activado");
      Serial.print("Temperatura actual: ");
      Serial.println(tempMem);
      Serial.print("Temperatura limite inferior: ");
      Serial.println(lowLimit);
      Serial.print("Temperatura limite superior: ");
      Serial.println(highLimit);
       Serial.println("--------------------------------------");
          }
        }


        publicarEstadoRele();
  }

/*
 * 
 * Función obtener Temperatura de la matriz en función al dia y la hora actual
 * Retorna un valor real con la temperatura
 * 
 */

 float getProgTemp(int hora , int dia){
  float temperaturaFromMatrix = tempMatrix[dia][hora];
  return temperaturaFromMatrix;
 }

/*
 * 
 * Función de actualización de la matriz con el valor actual de temperatura
 * La posición de la matriz es en función del dia y la hora
 * Solo modificará el valor de la matriz si se encuentra en el modo Auto
 * 
 */

void updateMatrix(){
  if(modo==0){
  int dia = getDia();
  int hora = getHora();

tempMatrix[dia][hora]=tempSP;
  }
 }

/*
 * 
 * Funcion para guardar valor actual de TempSP en la variable original
 * 
 */

void updateChangeTemp(){
  switch(modo){
    case 0:
    updateMatrix();
    break;

    case 1:
    tempDay=tempSP;
    break;

    case 2:
    tempNight=tempSP;
    break;

    case 3:
    break;
  }
 // publicarParamFloatValue(pathSetpoint, tempSP);           //publicar temperatura configurada como setpoint
}

/*
 * 
 * Función para actualizar los valores de la EEPROM con 
 * los valores actuales de la matriz de temperaturas de consigna
 * Se ha configurado que se haga el volcado en la EEPROM a las 4 de la madrugada
 * 
 */

void updateEEPROM(){
  int hora = getHora();
  int direccionInicio=0;
 
if((eepromUpdateEnbled==true)&& (hora==04)) {
  eepromUpdateEnbled=false;
    for ( int i = 0; i < 7; i++ ) {
        for( int j=0; j<24; j++){
            Serial.println("-----------escritura--------------");
            Serial.print("Valor de la matriz ");
            Serial.println(tempMatrix[i][j]);
            Serial.print(" Direccionamiento EEPROM ");
            Serial.println(direccionInicio);
            EEPROM.put(direccionInicio, tempMatrix[i][j]);
            delay(5);
            direccionInicio= direccionInicio + sizeof(tempMatrix[i][j]);
          }
        }
        //Guardamos valor offset
                  Serial.println("-----------escritura--------------");
          Serial.print("Valor de offset ");
          Serial.print(offset);
          Serial.print(" Direccionamiento EEPROM ");
          Serial.println(direccionInicio);
        EEPROM.put(direccionInicio, offset);
        direccionInicio= direccionInicio + sizeof(offset);
        delay(5);

         //Guardamos valor tempDay
                   Serial.println("-----------escritura--------------");
          Serial.print("Valor de tempDay ");
          Serial.print(tempDay);
          Serial.print(" Direccionamiento EEPROM ");
          Serial.println(direccionInicio);
        EEPROM.put(direccionInicio, tempDay);
          direccionInicio= direccionInicio + sizeof(tempDay);
        delay(5);

         //Guardamos valor tempNight
          Serial.println("-----------escritura--------------");
          Serial.print("Valor de tempNight ");
          Serial.print(tempNight);
          Serial.print(" Direccionamiento EEPROM ");
          Serial.println(direccionInicio);
        EEPROM.put(direccionInicio, tempNight);
        direccionInicio= direccionInicio + sizeof(tempNight);
        delay(5);
        
        EEPROM.commit();
       }
  }



 /*
   * Función publicar valor flotante a broker MQTT
   */
      void publicarParamFloatValue(String pathInput, float value){
    
        char valueToPublish[15];
        char targetPath[50];
        
      String strValue="";
      strValue= String(value,1);   
      pathInput.toCharArray(targetPath,50);
      strValue.toCharArray(valueToPublish, 15);
      client.publish(targetPath,valueToPublish);
        
      }

       /*
   * Función publicar valor String a broker MQTT
   */
      void publicarParamStringValue(String pathInput, String value){
    
        char valueToPublish[15];
        char targetPath[50];
        
      String strValue=value;
      pathInput.toCharArray(targetPath,50);
      strValue.toCharArray(valueToPublish, 15);
      client.publish(targetPath,valueToPublish);
        
      }

/*
 * Funcion obtener estado rele para publicar en broker MQTT
 */
 void publicarEstadoRele(){
                
        publicarParamStringValue(pathCaldera,releMem);
      }

//------------------------CALLBACK-----------------------------
void receivedCallback(char* topic, byte* payload, unsigned int length) {

  char PAYLOAD[15] = "    ";
  
  Serial.print("Mensaje Recibido: [");
  Serial.print(topic);
  Serial.print("] ");
  for (int i = 0; i < length; i++){
    PAYLOAD[i] = (char)payload[i];
  }
  Serial.println(PAYLOAD);

  if(String(topic)== pathSPSetup){
    tempSP = atof(PAYLOAD);
    Serial.println("Temperatura SP:");
    Serial.println(tempSP);
    publicarParamFloatValue(pathSetpoint, tempSP);           //publicar temperatura configurada como setpoint
    updateChangeTemp();
       }
   
   if(String(topic) == pathOffsetSetup){
    offset = atof(PAYLOAD);
    publicarParamFloatValue(pathOffset, offset);           //publicar offset del sensor
  }
 
   if(String(topic) == pathHystSetup){
    hysteresis = atof(PAYLOAD);
    publicarParamFloatValue(pathHyst, hysteresis);           //publicar offset del sensor
  }


  if(String(topic)== pathModoSetup){
    Serial.println("Modo actual:");
    Serial.println(modo);
    Serial.println("Modo recibido");
    modo = atoi(PAYLOAD);
    Serial.println(modo);

    currentMode(modo);
    publicarParamFloatValue(pathSetpoint, tempSP);           //publicar temperatura configurada como setpoint
    //publicarParamFloatValue(pathOffset, offset);             //publicar temperatura configurada como Offset
    modoText = currentMode(modo);                            //publicar modo
    publicarParamStringValue(pathModo,modoText);             //publicar modo
  }
  
  if(String(topic)== pathLectura){
    int valorPeticion = atoi(PAYLOAD);

    switch (valorPeticion) {  
   
      case 1: //pagina main.php
         printTemperatureToSerial();                             //publicar temperatura actual, mandamos a leer la temperatura antes usando la rutina
         publicarParamFloatValue(pathSetpoint, tempSP);           //publicar temperatura configurada como setpoint
         modoText = currentMode(modo);                            //publicar modo
         publicarParamStringValue(pathModo,modoText);             //publicar modo
         publicarEstadoRele();                                    //publicar estado rele, caldera
         break;

         case 2:  //pagina ajustes.php
         printTemperatureToSerial();                              //publicar temperatura actual, mandamos a leer la temperatura antes usando la rutina
         publicarParamFloatValue(pathSetpoint, tempSP);           //publicar temperatura configurada como setpoint
         modoText = currentMode(modo);                            //publicar modo
         publicarParamStringValue(pathModo,modoText);             //publicar modo
         publicarEstadoRele();                                    //publicar estado rele, caldera
         break;

         case 3: //ajustes avanzados
         publicarParamFloatValue(pathOffset, offset);             //publicar temperatura configurada como Offset
         publicarParamFloatValue(pathHyst, hysteresis);           //publicar valor hysteresis
         break;
    }
    
  }


  if(String(topic) == pathProgSetup){
   int i = atoi(PAYLOAD);
  
      for ( int j = 0; j < 24; j++ ) {
         String horaSeleccion = (String)j;                                                          //convertimos la hora en texto para concatenar topico
        String pathProg = "/"+ mqttUser+ "/" + "programacion" + "/" + horaSeleccion;                //topico para enviar el SP para programacion horaria
        Serial.println("----------------leer------------");
        Serial.print("i= ");
        Serial.print(i);
        Serial.print(" j= ");
        Serial.print(j);
        publicarParamFloatValue(pathProg, tempMatrix[i][j]);
        Serial.println("--------------------------------------------");    
        delay(5);
        }

  }

  if(String(topic)==pathPlanifSetup){
    char diaPlanifTemp[10]= "   ";
    diaPlanifTemp[0] = PAYLOAD[0]; 
    int diaPlanif= atoi(diaPlanifTemp); 
    Serial.print("Valor dia recibido ");Serial.println(diaPlanif);
    char horaTemp[10]= "    ";
    horaTemp[0] = PAYLOAD[1];   
    horaTemp[1] = PAYLOAD[2];
    int horaPlanif=atoi(horaTemp);
    Serial.print("Valor hora recibido ");Serial.println(horaPlanif);
    char valorTemp[12] = "     ";

      for (int i = 3; i < length; i++){
     valorTemp[(i-3)]  = PAYLOAD[i]; 
  }
    float valorPlanif = atof(valorTemp);
     Serial.print("Valor SP recibido ");Serial.println(valorPlanif);

       tempMatrix[diaPlanif][horaPlanif]=valorPlanif;
             
        
      }



}
 
/*
 * ------------------------RECONNECT-----------------------------
 */

void reconnect() {
 
  int retries = 3;
  // Loop hasta que estamos conectados
  while (!client.connected()) {
    Serial.print("Intentando conexion MQTT...");
    //Client ID
    String clientId= "ESP32Client";
      // Attempt to connect
      mqttUser.toCharArray(Nodo, 50);
      if (client.connect("ESP32Client", Nodo, mqttPassword)) {
      Serial.println("conectado");
      
   publicarParamFloatValue(pathSetpoint, tempSP);           //publicar temperatura configurada como setpoint
   publicarParamFloatValue(pathOffset, offset);             //publicar temperatura configurada como Offset
   modoText = currentMode(modo);
   publicarParamStringValue(pathModo,modoText);             //publicar modo de trabajo actual
   publicarParamFloatValue(pathHyst, hysteresis);         //publicar valor hysteresis
   client.subscribe("/smarttemp/setpoint");
   client.subscribe("/smarttemp/offset");
   client.subscribe("/smarttemp/peticion");
   client.subscribe("/smarttemp/modo");
   client.subscribe("/smarttemp/hysteresis");
   client.subscribe("/smarttemp/setup/hysteresis");
   client.subscribe("/smarttemp/setup/offset");
   client.subscribe("/smarttemp/setup/modo");
   client.subscribe("/smarttemp/setup/setpoint");
   client.subscribe("/smarttemp/setup/programacion");
   client.subscribe("/smarttemp/setup/planificacion");
     
    } else {
      Serial.print("fallo, rc=");
      Serial.print(client.state());
      Serial.println(" intenta nuevamente en 5 segundos");
      // espera 5 segundos antes de reintentar
      delay(5000);
    }
    retries--;
    if (retries == 0) {
      // esperar a que el WDT lo reinicie
     hard_restart();
        }
  }
 }



